import 'dart:convert';
import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:iversioning/iversioning.dart';

abstract class IVersioningBridgeInterface{
  Future<dynamic> getSupportedAppVersions({@required MethodCall call}); //Get supported app versions
  Future<dynamic> checkAppVersion({@required MethodCall call}); //Get supported app versions
  Future<dynamic> isLatestVersion({@required MethodCall call});
  Future<dynamic> getLatestAppVersion({@required MethodCall call});
}

class IVersioningBridge with IVersioningBridgeInterface {
  static const METHOD_CHANNEL = "com.inglab.bridging/iversion";
  static const platform = const MethodChannel(METHOD_CHANNEL);

  static const CHANNEL_INIT = "initialize";
  static const CHANNEL_SUPPORTED_APP_VERSIONS = "supportedAppVersion";
  static const CHANNEL_CHECK_APP_VERSION = "checkAppVersion";
  static const CHANNEL_IS_LATEST_VERSION = "isLatestVersion";
  static const CHANNEL_LATEST_APP_VERSION = "latestAppVersion";

  void initialize(){
    platform.setMethodCallHandler((call) async{
      dynamic result;
      try{
        switch(call.method){
          case CHANNEL_INIT: result = _initializeModule(call: call); break;
          case CHANNEL_SUPPORTED_APP_VERSIONS: result = getSupportedAppVersions(call: call); break;
          case CHANNEL_CHECK_APP_VERSION: result = checkAppVersion(call: call); break;
          case CHANNEL_IS_LATEST_VERSION: result = isLatestVersion(call: call); break;
          case CHANNEL_LATEST_APP_VERSION: result = getLatestAppVersion(call: call); break;
          default: return null;
        }
        platform.invokeMethod(call.method, jsonEncode(await result));
      }catch(e, s){
        platform.invokeMethod(call.method, jsonEncode({
          "success": false,
          "message": "Failed to process request with error : $e"
        }));
      }
      return true;
    });
  }

  IVersioning _getModuleInstance() => IVersioning.get();

  Future _initializeModule({MethodCall call}) async{
    final payload = jsonDecode(call.arguments);
    final baseUrl = payload["baseUrl"];
    final appVersion = payload["appVersion"];
    final apiKey = payload["apiKey"];
    final auth = IVersioningBuilder()
        .appVersion(appVersion)
        .baseUrl(baseUrl)
        .apiKey(apiKey)
        .build();

    return {
      "success": true,
      "message": "IVersioning initialized",
    };
  }

  @override
  Future checkAppVersion({@required MethodCall call}) async{
    final payload = jsonDecode(call.arguments);
    final takeAction = payload["takeAction"] == true;
    final result = await _getModuleInstance().checkAppVersion(null, takeAction: takeAction);

    return {
      "success": result.success,
      "message": result.message,
    };
  }

  @override
  Future getLatestAppVersion({@required MethodCall call}) async{
    final result = await _getModuleInstance().getLatestAppVersion();
    return {
      "success": result != null,
      "message": result != null ? "Get latest app version successful" : "Failed to get latest app version",
      "data": result.toJson()
    };
  }

  @override
  Future getSupportedAppVersions({@required MethodCall call}) async{
    final result = await _getModuleInstance().getSupportedAppVersions();
    return {
      "success": result.success,
      "message": result.message,
      "data": result.data.toJson()
    };
  }

  @override
  Future isLatestVersion({@required MethodCall call}) async{
    final result = await _getModuleInstance().isLatestVersion();
    return {
      "success": result != null,
      "message": result != null ? "Is latest version retrieved successful" : "Failed to check if this is the latest version",
      "data": result
    };
  }


}